@@ -26,6 +26,17 @@ if ! [ -x "$(command -v "${STEAM_ZENITY}" || true)" ]; then
     STEAM_ZENITY=
 fi
 
+log () {
+    echo "setup.sh[$$]: $*" >&2 || :
+}
+
+debug ()
+{
+    if [ -n "${STEAM_RUNTIME_VERBOSE-}" ]; then
+        log "$@"
+    fi
+}
+
 pin_newer_runtime_libs ()
 {
     # Set separator to newline just for this function
@@ -85,7 +96,7 @@ pin_newer_runtime_libs ()
     if [ "$zenity_progress" = true ]; then
         echo 4
     else
-        printf '\r 4%%    \r'
+        printf '\r 4%%    \r' >&2 || :
     fi
 
     if [[ -n "$identify_library_abi" ]]; then
@@ -105,9 +116,17 @@ pin_newer_runtime_libs ()
             abi=${line#*=}
 
             if [[ ${abi} == "i386-linux-gnu" ]]; then
-                host_libraries_32[$soname]=$soname_path
+                if [[ -n "${host_libraries_32[$soname]+isset}" ]]; then
+                    debug "Ignoring $soname_path because ${host_libraries_32[$soname]} is higher-precedence"
+                else
+                    host_libraries_32[$soname]=$soname_path
+                fi
             elif [[ ${abi} == "x86_64-linux-gnu" ]]; then
-                host_libraries_64[$soname]=$soname_path
+                if [[ -n "${host_libraries_64[$soname]+isset}" ]]; then
+                    debug "Ignoring $soname_path because ${host_libraries_64[$soname]} is higher-precedence"
+                else
+                    host_libraries_64[$soname]=$soname_path
+                fi
             fi
             # If it's not i386-linux-gnu nor x86_64-linux-gnu we just skip it
         done
@@ -119,7 +138,7 @@ pin_newer_runtime_libs ()
         if /sbin/ldconfig -XNv 2> /dev/null | mapfile -t ldconfig_output_array; then
             true  # OK
         else
-            >&2 echo "Warning: An unexpected error occurred while executing \"/sbin/ldconfig -XNv\", the exit status was $?"
+            log "Warning: An unexpected error occurred while executing \"/sbin/ldconfig -XNv\", the exit status was $?"
         fi
 
         ldconfig_num=${#ldconfig_output_array[@]}
@@ -130,7 +149,7 @@ pin_newer_runtime_libs ()
             if [ "$zenity_progress" = true ]; then
                 echo $(( ( 60 * n_done ) / ldconfig_num + 4 ))
             else
-                printf '\r %d%%    \r' $(( ( 60 * n_done ) / ldconfig_num + 4 ))
+                printf '\r %d%%    \r' $(( ( 60 * n_done ) / ldconfig_num + 4 )) >&2 || :
             fi
             (( n_done=n_done+1 ))
             # If line starts with a leading / and contains :, it's a new path prefix
@@ -165,10 +184,18 @@ pin_newer_runtime_libs ()
                 # Save into bitness-specific associative array with only SONAME as left-hand
                 if [[ $soname_details == *"32-bit"* ]]
                 then
-                    host_libraries_32[$soname]=$soname_fullpath
+                    if [[ -n "${host_libraries_32[$soname]+isset}" ]]; then
+                        debug "Ignoring $soname_fullpath because ${host_libraries_32[$soname]} is higher-precedence"
+                    else
+                        host_libraries_32[$soname]=$soname_fullpath
+                    fi
                 elif [[ $soname_details == *"64-bit"* ]]
                 then
-                    host_libraries_64[$soname]=$soname_fullpath
+                    if [[ -n "${host_libraries_64[$soname]+isset}" ]]; then
+                        debug "Ignoring $soname_fullpath because ${host_libraries_64[$soname]} is higher-precedence"
+                    else
+                        host_libraries_64[$soname]=$soname_fullpath
+                    fi
                 fi
             fi
         done
@@ -177,7 +204,7 @@ pin_newer_runtime_libs ()
     if [ "$zenity_progress" = true ]; then
         echo 64
     else
-        printf '\r 64%%    \r'
+        printf '\r 64%%    \r' >&2 || :
     fi
 
     mkdir "$steam_runtime_path/pinned_libs_32"
@@ -197,7 +224,7 @@ pin_newer_runtime_libs ()
         if [ "$zenity_progress" = true ]; then
             echo $(( ( 35 * n_done ) / find_num + 64 ))
         else
-            printf '\r %d%%    \r' $(( ( 35 * n_done ) / find_num + 64 ))
+            printf '\r %d%%    \r' $(( ( 35 * n_done ) / find_num + 64 )) >&2 || :
         fi
         (( n_done=n_done+1 ))
 
@@ -266,7 +293,7 @@ pin_newer_runtime_libs ()
 
         if [[ ! -f $host_library ]]; then continue; fi
 
-        #echo $soname ${host_libraries[$soname]} $r_lib_major $r_lib_minor $r_lib_third
+        #log $soname ${host_libraries[$soname]} $r_lib_major $r_lib_minor $r_lib_third
 
         # Pretty sure the host library already matches, but we need the rematch anyway
         if [[ ! $host_library =~ .*\.so.([[:digit:]]+).([[:digit:]]+).([[:digit:]]+)$ ]]; then continue; fi
@@ -329,9 +356,9 @@ pin_newer_runtime_libs ()
 
         # Print to stderr because zenity is consuming stdout
         if [[ $runtime_version_newer == "yes" ]]; then
-            >&2 echo "Found newer runtime version for $bitness-bit $soname. Host: $h_lib_major.$h_lib_minor.$h_lib_third Runtime: $r_lib_major.$r_lib_minor.$r_lib_third"
+            log "Found newer runtime version for $bitness-bit $soname. Host: $h_lib_major.$h_lib_minor.$h_lib_third Runtime: $r_lib_major.$r_lib_minor.$r_lib_third"
         elif [[ $runtime_version_newer == "forced" ]]; then
-            >&2 echo "Forced use of runtime version for $bitness-bit $soname. Host: $h_lib_major.$h_lib_minor.$h_lib_third Runtime: $r_lib_major.$r_lib_minor.$r_lib_third"
+            log "Forced use of runtime version for $bitness-bit $soname. Host: $h_lib_major.$h_lib_minor.$h_lib_third Runtime: $r_lib_major.$r_lib_minor.$r_lib_third"
         fi
 
         if [[ $runtime_version_newer == "yes" \
@@ -362,7 +389,7 @@ pin_newer_runtime_libs ()
     if [ "$zenity_progress" = true ]; then
         echo 100
     else
-        echo " 100%   "
+        echo " 100%   " >&2 || :
     fi
 }
 
@@ -424,7 +451,7 @@ check_pins ()
 
     if [[ $pins_need_redoing == "yes" ]]
     then
-        echo Updating Steam runtime environment...
+        log "Updating Steam runtime environment..."
         # Is always set at this point, but may be empty if the host lacks zenity
         if [ -n "${STEAM_ZENITY}" ]; then
             pin_newer_runtime_libs "$steam_runtime_path" | "${STEAM_ZENITY}" --progress --auto-close --percentage=0 --no-cancel --width 400 --title="Steam setup" --text="Updating Steam runtime environment..."
@@ -432,7 +459,7 @@ check_pins ()
             pin_newer_runtime_libs "$steam_runtime_path" "false"
         fi
     else
-        echo Steam runtime environment up-to-date!
+        log "Steam runtime environment up-to-date!"
     fi
 }
 
@@ -511,7 +538,7 @@ main ()
                 ;;
 
             (-*)
-                echo "$me: unknown option: $1" >&2
+                log "unknown option: $1"
                 usage 2
                 ;;
 
@@ -523,7 +550,7 @@ main ()
 
     if [ "$#" -gt 0 ]
     then
-        echo "$me does not have positional parameters" >&2
+        log "$me does not have positional parameters"
         usage 2
     fi
 
